Class: SimpleXDG

Inherits:
Object
  • Object
show all
Defined in:
lib/simple_xdg.rb,
lib/simple_xdg/version.rb

Overview

A class that provides tools for working with the XDG Base Directory Specification.

This class provides utility methods that locate base directories and search paths for application state, configuration, caches, and other data, according to the XDG Base Directory Spec version 0.8.

Example

require "simple_xdg"

xdg = SimpleXDG.new

# Get config file paths, in order from most to least important
config_files = xdg.lookup_config("my-config.toml")
config_files.each { |path| read_my_config(path) }

Windows operation

The Spec assumes a unix-like environment, and cannot be applied directly to Windows without modification. In general, this class will function on Windows, but with the following caveats:

  • All file paths must use Windows-style absolute paths, beginning with the drive letter.
  • Environment variables that can contain multiple paths (XDG_*_DIRS) use the Windows path delimiter (;) rather than the unix path delimiter (:).
  • Defaults for home directories (XDG_*_HOME) will follow unix conventions, using subdirectories under the user's profile directory rather than the Windows known folder paths.
  • Defaults for search paths (XDG_*_DIRS) will be empty and will not use the Windows known folder paths.

Defined Under Namespace

Classes: Error

Constant Summary collapse

VERSION =

Version of the simple_xdg gem

Returns:

  • (String)
"0.1.1"

Instance Method Summary collapse

Constructor Details

#initialize(env: ::ENV) ⇒ SimpleXDG

Create an instance of XDG.

Parameters:

  • env (Hash{String=>String}) (defaults to: ::ENV)

    the environment variables. Normally, you can omit this argument, as it will default to ::ENV.



52
53
54
55
# File 'lib/simple_xdg.rb', line 52

def initialize(env: ::ENV)
  require "fileutils"
  @env = env
end

Instance Method Details

#cache_homeString

Returns the absolute path to the single base directory relative to which user-specific non-essential (cached) data should be written.

Corresponds to the value of the $XDG_CACHE_HOME environment variable and its defaults according to the XDG Base Directory Spec.

Returns:

  • (String)


114
115
116
# File 'lib/simple_xdg.rb', line 114

def cache_home
  @cache_home ||= validate_dir_env("XDG_CACHE_HOME") || ::File.join(home_dir, ".cache")
end

#config_dirsArray<String>

Returns the set of preference ordered base directories relative to which configuration files should be searched, as an array of absolute paths. The array is ordered from most to least important, and does not include the config home directory.

Corresponds to the value of the $XDG_CONFIG_DIRS environment variable and its defaults according to the XDG Base Directory Spec.

Returns:

  • (Array<String>)


158
159
160
161
# File 'lib/simple_xdg.rb', line 158

def config_dirs
  @config_dirs ||= validate_dirs_env("XDG_CONFIG_DIRS") ||
                   validate_dirs(["/etc/xdg"])
end

#config_homeString

Returns the absolute path to the single base directory relative to which user-specific configuration files should be written.

Corresponds to the value of the $XDG_CONFIG_HOME environment variable and its defaults according to the XDG Base Directory Spec.

Returns:

  • (String)


88
89
90
# File 'lib/simple_xdg.rb', line 88

def config_home
  @config_home ||= validate_dir_env("XDG_CONFIG_HOME") || ::File.join(home_dir, ".config")
end

#data_dirsArray<String>

Returns the set of preference ordered base directories relative to which data files should be searched, as an array of absolute paths. The array is ordered from most to least important, and does not include the data home directory.

Corresponds to the value of the $XDG_DATA_DIRS environment variable and its defaults according to the XDG Base Directory Spec.

Returns:

  • (Array<String>)


142
143
144
145
# File 'lib/simple_xdg.rb', line 142

def data_dirs
  @data_dirs ||= validate_dirs_env("XDG_DATA_DIRS") ||
                 validate_dirs(["/usr/local/share", "/usr/share"])
end

#data_homeString

Returns the absolute path to the single base directory relative to which user-specific data files should be written.

Corresponds to the value of the $XDG_DATA_HOME environment variable and its defaults according to the XDG Base Directory Spec.

Returns:

  • (String)


75
76
77
# File 'lib/simple_xdg.rb', line 75

def data_home
  @data_home ||= validate_dir_env("XDG_DATA_HOME") || ::File.join(home_dir, ".local", "share")
end

#ensure_cache_subdir(path) ⇒ String

Returns the absolute path to a directory under #cache_home, creating it if it doesn't already exist.

Parameters:

  • path (String)

    The relative path to the subdir within the base cache directory.

Returns:

  • (String)

    The absolute path to the subdir.

Raises:

  • (SystemCallError)

    If a non-directory already exists there. It is unspecified which specific error will be raised; it typically could be Errno::EEXIST or Errno::ENOTDIR.



349
350
351
# File 'lib/simple_xdg.rb', line 349

def ensure_cache_subdir(path)
  ensure_subdir_internal(cache_home, path)
end

#ensure_config_subdir(path) ⇒ String

Returns the absolute path to a directory under #config_home, creating it if it doesn't already exist.

Parameters:

  • path (String)

    The relative path to the subdir within the base config directory.

Returns:

  • (String)

    The absolute path to the subdir.

Raises:

  • (SystemCallError)

    If a non-directory already exists there. It is unspecified which specific error will be raised; it typically could be Errno::EEXIST or Errno::ENOTDIR.



319
320
321
# File 'lib/simple_xdg.rb', line 319

def ensure_config_subdir(path)
  ensure_subdir_internal(config_home, path)
end

#ensure_data_subdir(path) ⇒ String

Returns the absolute path to a directory under #data_home, creating it if it doesn't already exist.

Parameters:

  • path (String)

    The relative path to the subdir within the base data directory.

Returns:

  • (String)

    The absolute path to the subdir.

Raises:

  • (SystemCallError)

    If a non-directory already exists there. It is unspecified which specific error will be raised; it typically could be Errno::EEXIST or Errno::ENOTDIR.



304
305
306
# File 'lib/simple_xdg.rb', line 304

def ensure_data_subdir(path)
  ensure_subdir_internal(data_home, path)
end

#ensure_state_subdir(path) ⇒ String

Returns the absolute path to a directory under #state_home, creating it if it doesn't already exist.

Parameters:

  • path (String)

    The relative path to the subdir within the base state directory.

Returns:

  • (String)

    The absolute path to the subdir.

Raises:

  • (SystemCallError)

    If a non-directory already exists there. It is unspecified which specific error will be raised; it typically could be Errno::EEXIST or Errno::ENOTDIR.



334
335
336
# File 'lib/simple_xdg.rb', line 334

def ensure_state_subdir(path)
  ensure_subdir_internal(state_home, path)
end

#executable_homeString

Returns the absolute path to the single base directory relative to which user-specific executable files may be written.

Returns the value of $HOME/.local/bin as specified by the XDG Base Directory Spec.

Returns:

  • (String)


127
128
129
# File 'lib/simple_xdg.rb', line 127

def executable_home
  @executable_home ||= ::File.join(home_dir, ".local", "bin")
end

#home_dirString

Returns the absolute path to the current user's home directory.

Returns:

  • (String)


62
63
64
# File 'lib/simple_xdg.rb', line 62

def home_dir
  @home_dir ||= validate_dir_env("HOME") || ::Dir.home
end

#lookup_cache(path, type: :file) ⇒ Array<String>

Searches the cache directory (#cache_home) for an object with the given relative path, and returns an array of zero or one absolute paths to any found object. Because the XDG basedir spec does not provide for a list of fallback directories for cache files (i.e. there is no XDG_CACHE_DIRS variable or list of default paths), this will return a maximum of one result. However, it returns an array for consistency with the #lookup_data and #lookup_config methods.

Parameters:

  • path (String)

    Relative path of the object to search for

  • type (String, Symbol, Array<String,Symbol>) (defaults to: :file)

    The type(s) of objects to find. You can specify any of the types defined by File::Stat#ftype, such as file or directory, or the special type any. Types can be specified as strings or the corresponding symbols. If this argument is not provided, the default of file is used.

Returns:

  • (Array<String>)


289
290
291
# File 'lib/simple_xdg.rb', line 289

def lookup_cache(path, type: :file)
  lookup_internal([cache_home], path, type)
end

#lookup_config(path, type: :file) ⇒ Array<String>

Searches the config directories for an object with the given relative path, and returns an array of absolute paths to all objects found in all config directories (i.e. #config_home and #config_dirs), in order from most to least important. Returns the empty array if no suitable objects are found.

If multiple objects are found, the caller should implement its own logic to resolve them. For example, it can select the first (most important) object, or implement logic to combine the contents.

Parameters:

  • path (String)

    Relative path of the object to search for

  • type (String, Symbol, Array<String,Symbol>) (defaults to: :file)

    The type(s) of objects to find. You can specify any of the types defined by File::Stat#ftype, such as file or directory, or the special type any. Types can be specified as strings or the corresponding symbols. If this argument is not provided, the default of file is used.

Returns:

  • (Array<String>)


245
246
247
# File 'lib/simple_xdg.rb', line 245

def lookup_config(path, type: :file)
  lookup_internal([config_home] + config_dirs, path, type)
end

#lookup_data(path, type: :file) ⇒ Array<String>

Searches the data directories for an object with the given relative path, and returns an array of absolute paths to all objects found in all data directories (i.e. #data_home and #data_dirs), in order from most to least important. Returns the empty array if no suitable objects are found.

If multiple objects are found, the caller should implement its own logic to resolve them. For example, it can select the first (most important) object, or implement logic to combine the contents.

Parameters:

  • path (String)

    Relative path of the object to search for

  • type (String, Symbol, Array<String,Symbol>) (defaults to: :file)

    The type(s) of objects to find. You can specify any of the types defined by File::Stat#ftype, such as file or directory, or the special type any. Types can be specified as strings or the corresponding symbols. If this argument is not provided, the default of file is used.

Returns:

  • (Array<String>)


221
222
223
# File 'lib/simple_xdg.rb', line 221

def lookup_data(path, type: :file)
  lookup_internal([data_home] + data_dirs, path, type)
end

#lookup_state(path, type: :file) ⇒ Array<String>

Searches the state directory (#state_home) for an object with the given relative path, and returns an array of zero or one absolute paths to any found object. Because the XDG basedir spec does not provide for a list of fallback directories for state files (i.e. there is no XDG_STATE_DIRS variable or list of default paths), this will return a maximum of one result. However, it returns an array for consistency with the #lookup_data and #lookup_config methods.

Parameters:

  • path (String)

    Relative path of the object to search for

  • type (String, Symbol, Array<String,Symbol>) (defaults to: :file)

    The type(s) of objects to find. You can specify any of the types defined by File::Stat#ftype, such as file or directory, or the special type any. Types can be specified as strings or the corresponding symbols. If this argument is not provided, the default of file is used.

Returns:

  • (Array<String>)


267
268
269
# File 'lib/simple_xdg.rb', line 267

def lookup_state(path, type: :file)
  lookup_internal([state_home], path, type)
end

#runtime_dirString?

Returns the absolute path to the single base directory relative to which user-specific runtime files and other file objects should be placed.

Corresponds to the value of the $XDG_RUNTIME_DIR environment variable according to the XDG Base Directory Spec.

Important: Returns nil if the $XDG_RUNTIME_DIR environment variable is unset or invalid. In such a case, it is the caller's responsibility to determine a fallback strategy, as this library cannot by itself implement a compliant fallback without OS help.

Returns:

  • (String, nil)


178
179
180
181
# File 'lib/simple_xdg.rb', line 178

def runtime_dir
  @runtime_dir = validate_dir_env("XDG_RUNTIME_DIR") unless defined? @runtime_dir
  @runtime_dir
end

#runtime_dir!String

Returns the absolute path to the single base directory relative to which user-specific runtime files and other file objects should be placed.

Corresponds to the value of the $XDG_RUNTIME_DIR environment variable according to the XDG Base Directory Spec.

Raises Error if the $XDG_RUNTIME_DIR environment variable is unset or invalid. Unlike #runtime_dir, does not return nil.

Returns:

  • (String)


197
198
199
# File 'lib/simple_xdg.rb', line 197

def runtime_dir!
  runtime_dir || raise(::SimpleXDG::Error, "XDG_RUNTIME_DIR is unset or invalid")
end

#state_homeString

Returns the absolute path to the single base directory relative to which user-specific state files should be written.

Corresponds to the value of the $XDG_STATE_HOME environment variable and its defaults according to the XDG Base Directory Spec.

Returns:

  • (String)


101
102
103
# File 'lib/simple_xdg.rb', line 101

def state_home
  @state_home ||= validate_dir_env("XDG_STATE_HOME") || ::File.join(home_dir, ".local", "state")
end